@@ -19,6 +19,7 @@
 
 #include <config.h>
 #include <time.h>
+#include <math.h>
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
@@ -29,20 +30,16 @@
 
 #include "main-window.h"
 #include "interface.h"
-#include "preferences-dialog.h"
-#include "search-dialog.h"
 #include "date-entry-dialog.h"
-#include "printing.h"
 #include "entry.h"
 #include "storage-manager.h"
 #include "event-manager.h"
 #include "event.h"
-#include "import-export-dialog.h"
 #include "uri-entry-dialog.h"
 #include "widgets/calendar.h"
 #include "widgets/calendar-button.h"
-#include "widgets/font-style-menu-action.h"
 #include "widgets/hyperlink-tag.h"
+#include "widgets/entry-tags-area.h"
 
 /* Interval for automatically saving the current entry. Currently an arbitrary 10 minutes. */
 #define SAVE_ENTRY_INTERVAL 10 * 60 /* seconds */
@@ -66,18 +63,11 @@ static void mw_italic_toggled_cb (GtkToggleAction *action, AlmanahMainWindow *ma
 static void mw_underline_toggled_cb (GtkToggleAction *action, AlmanahMainWindow *main_window);
 static void mw_hyperlink_toggled_cb (GtkToggleAction *action, AlmanahMainWindow *main_window);
 static void mw_events_updated_cb (AlmanahEventManager *event_manager, AlmanahEventFactoryType type_id, AlmanahMainWindow *main_window);
-static void mw_font_style_menu_position_func (GtkMenu *menu, int *x, int *y, gboolean *push_in, GtkMenuToolButton *button);
-static GtkMenuToolButton *mw_get_font_style_tool_button_from_action (GtkAction *action);
 static gboolean save_entry_timeout_cb (AlmanahMainWindow *self);
-static void mw_setup_toolbar (AlmanahMainWindow *main_window, AlmanahApplication *application, GtkToolbar *toolbar, GtkAction *today_action);
+static gint get_icon_margin (void);
+static void mw_setup_toolbar (AlmanahMainWindow *main_window, AlmanahApplication *application, GtkBuilder *builder);
 
 /* GtkBuilder callbacks */
-void mw_import_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
-void mw_export_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
-void mw_page_setup_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
-void mw_print_preview_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
-void mw_print_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
-void mw_quit_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
 void mw_cut_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
 void mw_copy_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
 void mw_paste_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
@@ -85,20 +75,18 @@ void mw_delete_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
 void mw_insert_time_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
 void mw_important_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
 void mw_select_date_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
-void mw_search_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
-void mw_preferences_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
-void mw_about_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
 void mw_jump_to_today_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
 void mw_old_entries_activate_cb (GtkAction *action, AlmanahMainWindow *main_window);
 void mw_events_tree_view_row_activated_cb (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, AlmanahMainWindow *main_window);
-void mw_font_style_activate_cb (AlmanahFontStyleMenuAction *action, AlmanahMainWindow *main_window);
 
 /* Other callbacks */
 void mw_calendar_day_selected_cb (AlmanahCalendarButton *calendar, AlmanahMainWindow *main_window);
+static void mw_menu_button_popup_visible_cb (GtkWidget *menu, GParamSpec *pspec, GtkWidget *button);
 
 struct _AlmanahMainWindowPrivate {
 	GtkTextView *entry_view;
 	GtkTextBuffer *entry_buffer;
+	AlmanahEntryTagsArea *entry_tags_area;
 	AlmanahCalendarButton *calendar_button;
 	GtkListStore *event_store;
 	GtkWidget *events_expander;
@@ -112,7 +100,6 @@ struct _AlmanahMainWindowPrivate {
 	GtkAction *copy_action;
 	GtkAction *delete_action;
 	GtkAction *important_action;
-	GtkMenuToolButton *font_style_tool_button;
 
 	gboolean updating_formatting;
 	gboolean pending_bold_active;
@@ -123,16 +110,13 @@ struct _AlmanahMainWindowPrivate {
 	gulong current_entry_notify_id; /* signal handler for current_entry::notify */
 	guint save_entry_timeout_id; /* source ID for timer to save current entry periodically */
 
-	GtkPrintSettings *print_settings;
-	GtkPageSetup *page_setup;
-
 #ifdef ENABLE_SPELL_CHECKING
 	GSettings *settings;
 	gulong spell_checking_enabled_changed_id; /* signal handler for application->settings::changed::spell-checking-enabled */
 #endif /* ENABLE_SPELL_CHECKING */
 };
 
-G_DEFINE_TYPE (AlmanahMainWindow, almanah_main_window, GTK_TYPE_WINDOW)
+G_DEFINE_TYPE (AlmanahMainWindow, almanah_main_window, GTK_TYPE_APPLICATION_WINDOW)
 #define ALMANAH_MAIN_WINDOW_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), ALMANAH_TYPE_MAIN_WINDOW, AlmanahMainWindowPrivate))
 
 static void
@@ -164,14 +148,6 @@ almanah_main_window_dispose (GObject *object)
 
 	set_current_entry (ALMANAH_MAIN_WINDOW (object), NULL);
 
-	if (priv->page_setup != NULL)
-		g_object_unref (priv->page_setup);
-	priv->page_setup = NULL;
-
-	if (priv->print_settings != NULL)
-		g_object_unref (priv->print_settings);
-	priv->print_settings = NULL;
-
 #ifdef ENABLE_SPELL_CHECKING
 	if (priv->settings != NULL) {
 		if (priv->spell_checking_enabled_changed_id != 0) {
@@ -195,9 +171,8 @@ almanah_main_window_new (AlmanahApplication *application)
 	AlmanahEventManager *event_manager;
 	AlmanahMainWindow *main_window;
 	AlmanahMainWindowPrivate *priv;
-	GtkToolbar *toolbar;
-	GtkAction *today_action;
 	GError *error = NULL;
+	AlmanahStorageManager *storage_manager;
 	const gchar *interface_filename = almanah_get_interface_filename ();
 	const gchar *object_names[] = {
 		"almanah_main_window",
@@ -244,6 +219,7 @@ almanah_main_window_new (AlmanahApplication *application)
 	/* Grab our child widgets */
 	priv->entry_view = GTK_TEXT_VIEW (gtk_builder_get_object (builder, "almanah_mw_entry_view"));
 	priv->entry_buffer = gtk_text_view_get_buffer (priv->entry_view);
+	priv->entry_tags_area = ALMANAH_ENTRY_TAGS_AREA (gtk_builder_get_object (builder, "almanah_mw_entry_tags_area"));
 	priv->event_store = GTK_LIST_STORE (gtk_builder_get_object (builder, "almanah_mw_event_store"));
 	priv->events_expander = GTK_WIDGET (gtk_builder_get_object (builder, "almanah_mw_events_expander"));
 	priv->events_count_label = GTK_LABEL (gtk_builder_get_object (builder, "almanah_mw_events_count_label"));
@@ -256,7 +232,6 @@ almanah_main_window_new (AlmanahApplication *application)
 	priv->copy_action = GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_copy"));
 	priv->delete_action = GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_delete"));
 	priv->important_action = GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_important"));
-	priv->font_style_tool_button = mw_get_font_style_tool_button_from_action (GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_font_style")));
 
 #ifdef ENABLE_SPELL_CHECKING
 	/* Set up spell checking, if it's enabled */
@@ -286,6 +261,13 @@ almanah_main_window_new (AlmanahApplication *application)
 	/* Similarly, make sure we're notified when there's a selection so we can change the status of cut/copy/paste actions */
 	g_signal_connect (priv->entry_buffer, "notify::has-selection", G_CALLBACK (mw_entry_buffer_has_selection_cb), main_window);
 
+	/* Set the storage to the tags area */
+	storage_manager = almanah_application_dup_storage_manager (application);
+	almanah_entry_tags_area_set_storage_manager (priv->entry_tags_area, storage_manager);
+	g_object_unref (storage_manager);
+	/* The entry GtkTextView is the widget that grab the focus after a tag was added */
+	almanah_entry_tags_area_set_back_widget (priv->entry_tags_area, GTK_WIDGET (priv->entry_view));
+
 	/* Connect up the formatting actions */
 	g_signal_connect (priv->bold_action, "toggled", G_CALLBACK (mw_bold_toggled_cb), main_window);
 	g_signal_connect (priv->italic_action, "toggled", G_CALLBACK (mw_italic_toggled_cb), main_window);
@@ -297,20 +279,8 @@ almanah_main_window_new (AlmanahApplication *application)
 	g_signal_connect (event_manager, "events-updated", G_CALLBACK (mw_events_updated_cb), main_window);
 	g_object_unref (event_manager);
 
-	/* Set up printing objects */
-	priv->print_settings = gtk_print_settings_new ();
-
-#ifdef GTK_PRINT_SETTINGS_OUTPUT_BASENAME
-	/* Translators: This is the default name of the PDF/PS/SVG file the diary is printed to if "Print to File" is chosen. */
-	gtk_print_settings_set (priv->print_settings, GTK_PRINT_SETTINGS_OUTPUT_BASENAME, _("Diary"));
-#endif
-
-	priv->page_setup = gtk_page_setup_new ();
-
-	/* Set up the toolbar */
-	toolbar = GTK_TOOLBAR (gtk_builder_get_object (builder, "almanah_mw_toolbar"));
-	today_action = GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_jump_to_today"));
-	mw_setup_toolbar (main_window, application, toolbar, today_action);
+	/* Set up the main toolbar */
+	mw_setup_toolbar (main_window, application, builder);
 
 	/* Select the current day and month */
 	mw_jump_to_today_activate_cb (NULL, main_window);
@@ -318,13 +288,6 @@ almanah_main_window_new (AlmanahApplication *application)
 	/* Set up a timeout for saving the current entry every so often. */
 	priv->save_entry_timeout_id = g_timeout_add_seconds (SAVE_ENTRY_INTERVAL, (GSourceFunc) save_entry_timeout_cb, main_window);
 
-#ifndef ENABLE_ENCRYPTION
-#ifndef ENABLE_SPELL_CHECKING
-	/* Remove the "Preferences" entry from the menu */
-	gtk_action_set_visible (GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_preferences")), FALSE);
-#endif /* !ENABLE_SPELL_CHECKING */
-#endif /* !ENABLE_ENCRYPTION */
-
 	g_object_unref (builder);
 
 	restore_window_state (main_window);
@@ -558,8 +521,8 @@ restore_window_state (AlmanahMainWindow *self)
 	g_object_unref (key_file_path);
 }
 
-static void
-save_current_entry (AlmanahMainWindow *self, gboolean prompt_user)
+void
+almanah_main_window_save_current_entry (AlmanahMainWindow *self, gboolean prompt_user)
 {
 	gboolean entry_exists, existing_entry_is_empty, entry_is_empty;
 	GDate date, last_edited;
@@ -673,7 +636,7 @@ done:
 static gboolean
 save_entry_timeout_cb (AlmanahMainWindow *self)
 {
-	save_current_entry (self, FALSE);
+	almanah_main_window_save_current_entry (self, FALSE);
 	return TRUE;
 }
 
@@ -814,7 +777,7 @@ mw_entry_buffer_has_selection_cb (GObject *object, GParamSpec *pspec, AlmanahMai
 static gboolean
 mw_delete_event_cb (GtkWindow *window, gpointer user_data)
 {
-	save_current_entry (ALMANAH_MAIN_WINDOW (window), TRUE);
+	almanah_main_window_save_current_entry (ALMANAH_MAIN_WINDOW (window), TRUE);
 	save_window_state (ALMANAH_MAIN_WINDOW (window));
 
 	gtk_widget_destroy (GTK_WIDGET (window));
@@ -822,92 +785,6 @@ mw_delete_event_cb (GtkWindow *window, gpointer user_data)
 	return TRUE;
 }
 
-void
-mw_import_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
-{
-	AlmanahApplication *application;
-	AlmanahStorageManager *storage_manager;
-	GtkWidget *dialog;
-
-	application = ALMANAH_APPLICATION (gtk_window_get_application (GTK_WINDOW (main_window)));
-	storage_manager = almanah_application_dup_storage_manager (application);
-	dialog = GTK_WIDGET (almanah_import_export_dialog_new (storage_manager, TRUE));
-	g_object_unref (storage_manager);
-
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (main_window));
-	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-
-	/* The dialog destroys itself once done */
-	gtk_widget_show_all (dialog);
-}
-
-void
-mw_export_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
-{
-	AlmanahApplication *application;
-	AlmanahStorageManager *storage_manager;
-	GtkWidget *dialog;
-
-	application = ALMANAH_APPLICATION (gtk_window_get_application (GTK_WINDOW (main_window)));
-	storage_manager = almanah_application_dup_storage_manager (application);
-	dialog = GTK_WIDGET (almanah_import_export_dialog_new (storage_manager, FALSE));
-	g_object_unref (storage_manager);
-
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (main_window));
-	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-
-	/* The dialog destroys itself once done */
-	gtk_widget_show_all (dialog);
-}
-
-void
-mw_page_setup_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
-{
-	AlmanahMainWindowPrivate *priv = main_window->priv;
-	GtkPageSetup *page_setup;
-
-	page_setup = gtk_print_run_page_setup_dialog (GTK_WINDOW (main_window), priv->page_setup, priv->print_settings);
-	if (priv->page_setup != NULL)
-		g_object_unref (priv->page_setup);
-	priv->page_setup = page_setup;
-}
-
-void
-mw_print_preview_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
-{
-	AlmanahMainWindowPrivate *priv = main_window->priv;
-	AlmanahApplication *application;
-	AlmanahStorageManager *storage_manager;
-
-	application = ALMANAH_APPLICATION (gtk_window_get_application (GTK_WINDOW (main_window)));
-	storage_manager = almanah_application_dup_storage_manager (application);
-	almanah_print_entries (TRUE, GTK_WINDOW (main_window), &(priv->page_setup), &(priv->print_settings), storage_manager);
-	g_object_unref (storage_manager);
-}
-
-void
-mw_print_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
-{
-	AlmanahMainWindowPrivate *priv = main_window->priv;
-	AlmanahApplication *application;
-	AlmanahStorageManager *storage_manager;
-
-	application = ALMANAH_APPLICATION (gtk_window_get_application (GTK_WINDOW (main_window)));
-	storage_manager = almanah_application_dup_storage_manager (application);
-	almanah_print_entries (FALSE, GTK_WINDOW (main_window), &(priv->page_setup), &(priv->print_settings), storage_manager);
-	g_object_unref (storage_manager);
-}
-
-void
-mw_quit_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
-{
-	/* Hide the window to make things look faster */
-	gtk_widget_hide (GTK_WIDGET (main_window));
-
-	save_current_entry (main_window, TRUE);
-	gtk_widget_destroy (GTK_WIDGET (main_window));
-}
-
 void
 mw_cut_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 {
@@ -957,6 +834,8 @@ mw_select_date_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 {
 	AlmanahDateEntryDialog *dialog = almanah_date_entry_dialog_new ();
 
+	almanah_calendar_button_popdown (main_window->priv->calendar_button);
+
 	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (main_window));
 	gtk_widget_show_all (GTK_WIDGET (dialog));
 	if (almanah_date_entry_dialog_run (dialog) == TRUE) {
@@ -970,39 +849,6 @@ mw_select_date_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
-void
-mw_search_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
-{
-	AlmanahSearchDialog *dialog = almanah_search_dialog_new ();
-
-	gtk_window_set_application (GTK_WINDOW (dialog), gtk_window_get_application (GTK_WINDOW (main_window)));
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (main_window));
-	gtk_widget_show (GTK_WIDGET (dialog));
-	gtk_dialog_run (GTK_DIALOG (dialog));
-
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-}
-
-void
-mw_preferences_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
-{
-#if defined(ENABLE_ENCRYPTION) || defined(ENABLE_SPELL_CHECKING)
-	AlmanahApplication *application;
-	GSettings *settings;
-	AlmanahPreferencesDialog *dialog;
-
-	application = ALMANAH_APPLICATION (gtk_window_get_application (GTK_WINDOW (main_window)));
-	settings = almanah_application_dup_settings (application);
-	dialog = almanah_preferences_dialog_new (settings);
-	g_object_unref (settings);
-
-	gtk_widget_show_all (GTK_WIDGET (dialog));
-	gtk_dialog_run (GTK_DIALOG (dialog));
-
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-#endif /* ENABLE_ENCRYPTION || ENABLE_SPELL_CHECKING */
-}
-
 static void
 apply_formatting (AlmanahMainWindow *self, const gchar *tag_name, gboolean applying)
 {
@@ -1161,67 +1007,6 @@ mw_hyperlink_toggled_cb (GtkToggleAction *action, AlmanahMainWindow *self)
 	gtk_text_buffer_set_modified (priv->entry_buffer, TRUE);
 }
 
-void
-mw_about_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
-{
-	AlmanahApplication *application;
-	AlmanahStorageManager *storage_manager;
-	gchar *license, *description;
-	guint entry_count;
-
-	const gchar *authors[] =
-	{
-		"Philip Withnall <philip@tecnocode.co.uk>",
-		NULL
-	};
-	const gchar *license_parts[] = {
-		N_("Almanah is free software: you can redistribute it and/or modify "
-		   "it under the terms of the GNU General Public License as published by "
-		   "the Free Software Foundation, either version 3 of the License, or "
-		   "(at your option) any later version."),
-		N_("Almanah is distributed in the hope that it will be useful, "
-		   "but WITHOUT ANY WARRANTY; without even the implied warranty of "
-		   "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
-		   "GNU General Public License for more details."),
-		N_("You should have received a copy of the GNU General Public License "
-		   "along with Almanah.  If not, see <http://www.gnu.org/licenses/>."),
-	};
-
-	license = g_strjoin ("\n\n",
-			  _(license_parts[0]),
-			  _(license_parts[1]),
-			  _(license_parts[2]),
-			  NULL);
-
-	application = ALMANAH_APPLICATION (gtk_window_get_application (GTK_WINDOW (main_window)));
-	storage_manager = almanah_application_dup_storage_manager (application);
-	almanah_storage_manager_get_statistics (storage_manager, &entry_count);
-	g_object_unref (storage_manager);
-
-	description = g_strdup_printf (_("A helpful diary keeper, storing %u entries."), entry_count);
-
-	gtk_show_about_dialog (GTK_WINDOW (main_window),
-				"version", VERSION,
-				"copyright", _("Copyright \xc2\xa9 2008-2009 Philip Withnall"),
-				"comments", description,
-				"authors", authors,
-				/* Translators: please include your names here to be credited for your hard work!
-				 * Format:
-				 * "Translator name 1 <translator@email.address>\n"
-				 * "Translator name 2 <translator2@email.address>"
-				 */
-				"translator-credits", _("translator-credits"),
-				"logo-icon-name", "almanah",
-				"license", license,
-				"wrap-license", TRUE,
-				"website-label", _("Almanah Website"),
-				"website", "http://live.gnome.org/Almanah_Diary",
-				NULL);
-
-	g_free (license);
-	g_free (description);
-}
-
 void
 mw_jump_to_today_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 {
@@ -1233,103 +1018,10 @@ mw_jump_to_today_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 void
 mw_old_entries_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 {
-	// TODO: Show the old entries
+	/* TODO: Show the old entries */
 	g_debug ("Old entries clicked, but nothing implemented yet...");
 }
 
-/**
- * Just copied from GtkMenuToolButton (menu_position_func),
- * but using the button widget (not the arrow widget) for
- * menu position calc, and removing the code for a vertical
- * toolbar situation.
- *
- * Copyright (C) 2003 Ricardo Fernandez Pascual
- * Copyright (C) 2004 Paolo Borelli
- * Licensed under the GPL v2
- */
-static void
-mw_font_style_menu_position_func (GtkMenu *menu, int *x, int *y, gboolean *push_in, GtkMenuToolButton *button)
-{
-	GtkAllocation arrow_allocation;
-	GtkWidget *widget = GTK_WIDGET (button);
-	GtkRequisition menu_req;
-	GtkTextDirection direction;
-	GdkRectangle monitor;
-	gint monitor_num;
-	GdkScreen *screen;
-	GdkWindow *window;
-	GtkAllocation allocation;
-
-	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &menu_req, NULL);
-	direction = gtk_widget_get_direction (widget);
-	window = gtk_widget_get_window (widget);
-
-	screen = gtk_widget_get_screen (GTK_WIDGET (menu));
-	monitor_num = gdk_screen_get_monitor_at_window (screen, window);
-	if (monitor_num < 0)
-		monitor_num = 0;
-	gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
-
-	gtk_widget_get_allocation (widget, &allocation);
-
-	gdk_window_get_origin (window, x, y);
-	*x += allocation.x;
-	*y += allocation.y;
-
-	if (direction == GTK_TEXT_DIR_LTR)
-		*x += MAX (allocation.width - menu_req.width, 0);
-	else if (menu_req.width > allocation.width)
-		*x -= menu_req.width - allocation.width;
-
-	if ((*y + arrow_allocation.height + menu_req.height) <= monitor.y + monitor.height)
-		*y += allocation.height;
-	else if ((*y - menu_req.height) >= monitor.y)
-		*y -= menu_req.height;
-	else if (monitor.y + monitor.height - (*y + arrow_allocation.height) > *y)
-		*y += allocation.height;
-	else
-		*y -= menu_req.height;
-
-	*push_in = FALSE;
-}
-
-void
-mw_font_style_activate_cb (AlmanahFontStyleMenuAction *action, AlmanahMainWindow *main_window)
-{
-	AlmanahMainWindowPrivate *priv = ALMANAH_MAIN_WINDOW_GET_PRIVATE(main_window);
-	GtkMenu *menu = NULL;
-
-	if (GTK_IS_MENU_TOOL_BUTTON (priv->font_style_tool_button)) {
-		menu = GTK_MENU (gtk_menu_tool_button_get_menu (priv->font_style_tool_button));
-	}
-
-	if (GTK_IS_MENU (menu)) {
-		gtk_menu_popup (menu, NULL, NULL,
-				(GtkMenuPositionFunc) mw_font_style_menu_position_func,
-				priv->font_style_tool_button,
-				0,
-				gtk_get_current_event_time());
-	}
-}
-
-static GtkMenuToolButton *
-mw_get_font_style_tool_button_from_action (GtkAction *action)
-{
-	GSList *proxies;
-	GtkMenuToolButton *tool_button = NULL;
-
-	proxies = gtk_action_get_proxies (GTK_ACTION (action));
-	while (proxies != NULL) {
-		if (GTK_IS_MENU_TOOL_BUTTON (proxies->data)) {
-			tool_button = GTK_MENU_TOOL_BUTTON (proxies->data);
-		}
-
-		proxies = proxies->next;
-	}
-
-	return tool_button;
-}
-
 static void
 clear_factory_events (AlmanahMainWindow *self, AlmanahEventFactoryType type_id)
 {
@@ -1382,21 +1074,26 @@ mw_events_updated_cb (AlmanahEventManager *event_manager, AlmanahEventFactoryTyp
 	for (events = _events; events != NULL; events = g_slist_next (events)) {
 		GtkTreeIter iter;
 		AlmanahEvent *event = events->data;
+		gchar *event_time;
 
 		g_debug ("\t%s", almanah_event_format_value (event));
 
+		/* Translators: this is an event source name (like Calendar appointment) and the time when the event takes place */
+		event_time = g_strdup_printf (_("%s @ %s"), almanah_event_format_time (event), almanah_event_get_name (event));
+
 		gtk_list_store_append (priv->event_store, &iter);
 		gtk_list_store_set (priv->event_store, &iter,
 				    0, event,
 				    1, almanah_event_get_icon_name (event),
 				    2, type_id,
 				    3, almanah_event_format_value (event),
-				    4, g_strdup_printf ("<small>%s @ %s</small>", almanah_event_format_time (event), almanah_event_get_name (event)),
+				    4, g_strdup_printf ("<small>%s</small>", event_time),
 				    -1);
 
 		events_count++;
 
 		g_object_unref (event);
+		g_free (event_time);
 	}
 
 	events_text = g_strdup_printf ("%u", events_count);
@@ -1423,7 +1120,7 @@ mw_calendar_day_selected_cb (AlmanahCalendarButton *calendar_button, AlmanahMain
 	AlmanahEventManager *event_manager;
 	GDate calendar_date;
 #ifdef ENABLE_SPELL_CHECKING
-	GtkSpell *gtkspell;
+	GtkSpellChecker *gtkspell;
 #endif /* ENABLE_SPELL_CHECKING */
 	AlmanahMainWindowPrivate *priv = main_window->priv;
 	AlmanahEntry *entry;
@@ -1432,7 +1129,7 @@ mw_calendar_day_selected_cb (AlmanahCalendarButton *calendar_button, AlmanahMain
 	application = ALMANAH_APPLICATION (gtk_window_get_application (GTK_WINDOW (main_window)));
 
 	/* Save the previous entry */
-	save_current_entry (main_window, TRUE);
+	almanah_main_window_save_current_entry (main_window, TRUE);
 
 	/* Update the date label */
 	almanah_calendar_button_get_date (main_window->priv->calendar_button, &calendar_date);
@@ -1480,9 +1177,9 @@ mw_calendar_day_selected_cb (AlmanahCalendarButton *calendar_button, AlmanahMain
 
 #ifdef ENABLE_SPELL_CHECKING
 	/* Ensure the spell-checking is updated */
-	gtkspell = gtkspell_get_from_text_view (priv->entry_view);
+	gtkspell = gtk_spell_checker_get_from_text_view (priv->entry_view);
 	if (gtkspell != NULL) {
-		gtkspell_recheck_all (gtkspell);
+		gtk_spell_checker_recheck_all (gtkspell);
 		gtk_widget_queue_draw (GTK_WIDGET (priv->entry_view));
 	}
 #endif /* ENABLE_SPELL_CHECKING */
@@ -1494,6 +1191,9 @@ mw_calendar_day_selected_cb (AlmanahCalendarButton *calendar_button, AlmanahMain
 	event_manager = almanah_application_dup_event_manager (application);
 	almanah_event_manager_query_events (event_manager, ALMANAH_EVENT_FACTORY_UNKNOWN, &calendar_date);
 	g_object_unref (event_manager);
+
+	/* Show the entry tags */
+	almanah_entry_tags_area_set_entry (priv->entry_tags_area, priv->current_entry);
 }
 
 void
@@ -1512,18 +1212,60 @@ mw_events_tree_view_row_activated_cb (GtkTreeView *tree_view, GtkTreePath *path,
 }
 
 static void
-mw_setup_toolbar (AlmanahMainWindow *main_window, AlmanahApplication *application, GtkToolbar *toolbar, GtkAction *today_action)
+mw_menu_button_popup_visible_cb (GtkWidget *menu, GParamSpec *pspec, GtkWidget *button)
 {
-	GtkToolItem *calendar_button_item, *separator;
+	/* Set on/off the active menu style when the menu is visible */
+	if (gtk_widget_get_visible (menu))
+		gtk_style_context_add_class (gtk_widget_get_style_context (button), "active-menu");
+	else
+		gtk_style_context_remove_class (gtk_widget_get_style_context (button), "active-menu");
+}
+
+static gint
+get_icon_margin (void)
+{
+  gint toolbar_size, menu_size;
+
+  gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &menu_size, NULL);
+  gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &toolbar_size, NULL);
+  return (gint) ((toolbar_size - menu_size) / 2);
+}
+
+static void
+mw_setup_toolbar (AlmanahMainWindow *main_window, AlmanahApplication *application, GtkBuilder *builder)
+{
+	GtkToolbar *toolbar;
+	GtkToolItem *tool_item;
+	GtkWidget *button, *button_image, *popup;
+	GtkUIManager *manager;
 	AlmanahStorageManager *storage_manager;
 
-	/* Insert a dynamic space between the text style and calendar & important.
-	 * This can't be done using the <separator/> in the UI file at the moment
+	toolbar = GTK_TOOLBAR (gtk_builder_get_object (builder, "almanah_mw_toolbar"));
+	manager = GTK_UI_MANAGER (gtk_builder_get_object (builder, "almanah_ui_manager"));
+
+	/* Allow drag the window using the toolbar */
+	gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (toolbar)), GTK_STYLE_CLASS_MENUBAR);
+	gtk_toolbar_set_icon_size (toolbar, GTK_ICON_SIZE_MENU);
+
+	/* The text style button */
+	tool_item = gtk_tool_item_new ();
+	button = gtk_menu_button_new ();
+	popup = gtk_ui_manager_get_widget (manager, "/almanah_mw_font_menu");
+	gtk_menu_button_set_popup (GTK_MENU_BUTTON (button), popup);
+	button_image = gtk_image_new_from_icon_name ("gtk-select-font", GTK_ICON_SIZE_MENU);
+	g_object_set (button_image, "margin", get_icon_margin (), NULL);
+	gtk_button_set_image (GTK_BUTTON (button), button_image);
+	gtk_container_add (GTK_CONTAINER (tool_item), button);
+	g_signal_connect (popup, "notify::visible", G_CALLBACK (mw_menu_button_popup_visible_cb), button);
+	gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (tool_item));
+
+	/* Insert a dynamic space between the insert link and calendar & important.
+	 * This can't be done using the <tool_item/> in the UI file at the moment
 	 */
-	separator = gtk_separator_tool_item_new ();
-	gtk_separator_tool_item_set_draw (GTK_SEPARATOR_TOOL_ITEM (separator), FALSE);
-	gtk_tool_item_set_expand (separator, TRUE);
-	gtk_toolbar_insert (toolbar, separator, 2);
+	tool_item = gtk_separator_tool_item_new ();
+	gtk_separator_tool_item_set_draw (GTK_SEPARATOR_TOOL_ITEM (tool_item), FALSE);
+	gtk_tool_item_set_expand (tool_item, TRUE);
+	gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (tool_item));
 
 	/* Setup the calendar button */
 	storage_manager = almanah_application_dup_storage_manager (application);
@@ -1531,13 +1273,38 @@ mw_setup_toolbar (AlmanahMainWindow *main_window, AlmanahApplication *applicatio
 	g_object_unref (storage_manager);
 	g_signal_connect (main_window->priv->calendar_button, "day-selected", G_CALLBACK (mw_calendar_day_selected_cb), main_window);
 	/* Use the same action for the today button in the dropdown window */
-	almanah_calendar_button_set_today_action (main_window->priv->calendar_button, today_action);
-
+	almanah_calendar_button_set_today_action (main_window->priv->calendar_button, GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_jump_to_today")));
+	almanah_calendar_button_set_select_date_action (main_window->priv->calendar_button, GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_select_date")));
 	/* Insert the calendar button into the toolbar through a GtkToolItem but button style */
-	calendar_button_item = gtk_tool_item_new ();
-	gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (calendar_button_item)), GTK_STYLE_CLASS_RAISED);
-	gtk_container_add (GTK_CONTAINER (calendar_button_item), GTK_WIDGET (main_window->priv->calendar_button));
-	gtk_toolbar_insert (toolbar, calendar_button_item, 3);
+	tool_item = gtk_tool_item_new ();
+	gtk_container_add (GTK_CONTAINER (tool_item), GTK_WIDGET (main_window->priv->calendar_button));
+	gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (tool_item));
+
+	/* Important entry */
+	tool_item = gtk_separator_tool_item_new ();
+	gtk_separator_tool_item_set_draw (GTK_SEPARATOR_TOOL_ITEM (tool_item), FALSE);
+	gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (tool_item));
+	tool_item = gtk_toggle_tool_button_new ();
+	gtk_activatable_set_related_action (GTK_ACTIVATABLE (tool_item), GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_important")));
+	gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (tool_item));
+
+	/* Another dynamic tool_item */
+	tool_item = gtk_separator_tool_item_new ();
+	gtk_separator_tool_item_set_draw (GTK_SEPARATOR_TOOL_ITEM (tool_item), FALSE);
+	gtk_tool_item_set_expand (tool_item, TRUE);
+	gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (tool_item));
+
+	/* Menu button with the common GNOME applications style (Nautilus, Epiphany, ...) */
+	tool_item = gtk_tool_item_new ();
+	button = gtk_menu_button_new ();
+	popup = gtk_ui_manager_get_widget (manager, "/almanah_mw_menu_button");
+	gtk_menu_button_set_popup (GTK_MENU_BUTTON (button), popup);
+	button_image = gtk_image_new_from_icon_name ("emblem-system-symbolic", GTK_ICON_SIZE_MENU);
+	g_object_set (button_image, "margin", get_icon_margin (), NULL);
+	gtk_button_set_image (GTK_BUTTON (button), button_image);
+	gtk_container_add (GTK_CONTAINER (tool_item), button);
+	gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (tool_item));
+	g_signal_connect (popup, "notify::visible", G_CALLBACK (mw_menu_button_popup_visible_cb), button);
 }
 
 #ifdef ENABLE_SPELL_CHECKING
@@ -1573,13 +1340,13 @@ enable_spell_checking (AlmanahMainWindow *self, GError **error)
 {
 	AlmanahApplication *application;
 	GSettings *settings;
-	GtkSpell *gtkspell;
+	GtkSpellChecker *gtkspell;
 	gchar *spelling_language;
 	GtkTextTagTable *table;
 	GtkTextTag *tag;
 
 	/* Bail out if spell checking's already enabled */
-	if (gtkspell_get_from_text_view (self->priv->entry_view) != NULL)
+	if (gtk_spell_checker_get_from_text_view (self->priv->entry_view) != NULL)
 		return TRUE;
 
 	/* If spell checking wasn't already enabled, we have a dummy gtkspell-misspelled text tag to destroy */
@@ -1600,7 +1367,9 @@ enable_spell_checking (AlmanahMainWindow *self, GError **error)
 		spelling_language = NULL;
 	}
 
-	gtkspell = gtkspell_new_attach (self->priv->entry_view, spelling_language, error);
+	gtkspell = gtk_spell_checker_new ();
+	gtk_spell_checker_set_language (gtkspell, spelling_language, error);
+	gtk_spell_checker_attach (gtkspell, self->priv->entry_view);
 	g_free (spelling_language);
 
 	if (gtkspell == NULL)
@@ -1611,13 +1380,13 @@ enable_spell_checking (AlmanahMainWindow *self, GError **error)
 static void
 disable_spell_checking (AlmanahMainWindow *self)
 {
-	GtkSpell *gtkspell;
+	GtkSpellChecker *gtkspell;
 	GtkTextTagTable *table;
 	GtkTextTag *tag;
 
-	gtkspell = gtkspell_get_from_text_view (self->priv->entry_view);
+	gtkspell = gtk_spell_checker_get_from_text_view (self->priv->entry_view);
 	if (gtkspell != NULL)
-		gtkspell_detach (gtkspell);
+		gtk_spell_checker_detach (gtkspell);
 
 	/* Remove the old gtkspell-misspelling text tag */
 	table = gtk_text_buffer_get_tag_table (self->priv->entry_buffer);
